/***************************************************************************************************
*             C# sample for the usage of class TopoART_C (simple classification task)              *
****************************************************************************************************
*                           Created by Marko Tscherepanow, 22 December 2016                        *
***************************************************************************************************/

// Compile and run from the console: dotnet run --project TopoART-C_sample1.csproj

using System;
using LibTopoART;

namespace LibTopoART_samples
{
	/// <summary>
	/// Simple classification sample. [C#]
	/// <para>
	/// This sample demonstrates training and several possibilities for prediction at the example of
	/// a simple classification task.
	/// </para>
	/// </summary>
	class TopoART_C_sample1
	{
		static void Main() {
			long i;

			// Training data
			decimal[][] training_samples = {new decimal[] {0.1m, 0.1m},
											new decimal[] {0.15m, 0.2m},
											new decimal[] {0.21m, 0.11m},
											new decimal[] {0.9m, 0.1m},
											new decimal[] {0.85m, 0.12m}};
			long[] training_sample_class_IDs = {1, 1, 1, 2, 2};

			// Test data
			decimal[][] test_samples = {new decimal[] {0.09m, 0.11m},
										new decimal[] {0.91m, 0.13m},
										new decimal[] {0.9m, 0.75m}};

			// Create TopoART-C network using rho_a = 0.90 and default values for the remaining parameters
			TopoART_C tac = new TopoART_C(2, 2, 0.9m);

			// Training
			for(i = 0; i < training_samples.Length; ++i)
				tac.Learn(training_samples[i], training_sample_class_IDs[i]);

			// Prediction with nu = 5 (higher values cannot change the results in this sample, independent of rho_a)
			long class_ID1 = tac.Predict(test_samples[0], 5);	// belongs to class 1
			if(class_ID1 == 1)
				Console.WriteLine("Correctly classified the first test sample");
			long class_ID2 = tac.Predict(test_samples[1], 5);	// belongs to class 2
			if(class_ID2 == 2)
				Console.WriteLine("Correctly classified the second test sample");

			// The third test sample belongs neither to class 1 nor to class 2. However, it is closer to class 2 and
			// differs from it only in the second input dimension. This is confirmed by the following predictions.

			// Extended prediction with nu = 5 and an empty mask vector
			TopoART_C_prediction pred3 = tac.Predict(test_samples[2], null, 5);	
			Console.WriteLine("Predicted class ID {0} for the third test sample (all dimensions), but the confidence of {1} signifies some uncertainty",
				pred3.class_ID, pred3.confidence);

			// Extended prediction with nu = 5 and a mask vector inhibiting the second input dimension
			bool[] mask_vector = {false, true};
			TopoART_C_prediction pred4 = tac.Predict(test_samples[2], mask_vector, 5);
			Console.WriteLine("Predicted class ID {0} for the third test sample (only first dimension) with the confidence of {1} signifying complete certainty",
				pred4.class_ID, pred4.confidence);

		}
	}
}